package cz.jzitnik.game.logic.services.flowing;

import java.util.Random;

import cz.jzitnik.game.Game;
import cz.jzitnik.game.annotations.CustomLogic;
import cz.jzitnik.game.annotations.Flamable;
import cz.jzitnik.game.entities.Block;
import cz.jzitnik.game.logic.CustomLogicInterface;
import cz.jzitnik.tui.ScreenRenderer;

@CustomLogic
public class LavaFireLogic implements CustomLogicInterface {
    private static final int RADIUS = 30;
    private Random random = new Random();

    @Override
    public void nextIteration(Game game, ScreenRenderer ignored) {
        int[] data = game.getPlayerCords();
        var world = game.getWorld();
        int playerX = data[0];
        int playerY = data[1];

        int startX = Math.max(0, playerX - RADIUS);
        int startY = Math.max(0, playerY - RADIUS);
        int endX = Math.min(world[0].length - 1, playerX + RADIUS);
        int endY = Math.min(world.length - 1, playerY + RADIUS);

        for (int y = startY; y <= endY; y++) {
            for (int x = startX; x <= endX; x++) {
                var blocks = world[y][x];

                if (blocks.stream().anyMatch(block -> block.getBlockId().equals("lava"))) {
                    setOnFire(game, x + 1, y);
                    setOnFire(game, x - 1, y);
                    setOnFire(game, x, y + 1);
                    setOnFire(game, x, y - 1);
                }
            }
        }
    }

    public void setOnFire(Game game, int x, int y) {
        var world = game.getWorld();

        if (x < 0 || y < 0 || y >= world.length || x >= world[0].length) {
            return;
        }

        for (Block block : world[y][x]) {
            if (block.getClass().isAnnotationPresent(Flamable.class) && !block.isOnFire()) {
                int maxTime = random.nextInt(8) + 5;
                block.setBurningTime(block.getBurningTime() + 1);
                if (block.getBurningTime() >= maxTime) {
                    block.setOnFire(true);
                    block.setBurningTime(0);
                }
            }
        }
    }
}
