package cz.jzitnik.tui;

import java.util.HashMap;
import java.util.Optional;

public abstract class Sprite<E extends Enum<E>> {
    private HashMap<E, String> resourcesLocation;
    private HashMap<E, Optional<String>> resources;
    private String defaultResource;
    private String defaultResourceLocation;
    private Class<E> enumClass;

    protected final void loadResources(HashMap<E, String> values, Class<E> enumClass) {
        resources = new HashMap<>();
        resourcesLocation = new HashMap<>();
        for (E key : values.keySet()) {
            var value = values.get(key);
            resourcesLocation.put(key, value);
            resources.put(key, Optional.empty());
        }

        this.enumClass = enumClass;
    }

    private String loadResource(E key) {
        var location = resourcesLocation.get(key);
        var resource = ResourceLoader.loadResource(location);

        resources.put(key, Optional.of(resource));
        resourcesLocation.remove(key);

        return resource;
    }

    protected final void loadResource(String resource) {
        defaultResourceLocation = resource;
    }

    private final String loadResource() {
        var resource = ResourceLoader.loadResource(defaultResourceLocation);

        defaultResource = resource;

        return resource;
    }

    protected final String getResource(E key) {
        var resource = resources.get(key);

        if (resource.isEmpty()) {
            return loadResource(key);
        } else {
            return resource.get();
        }
    }

    protected final String getResource() {
        if (defaultResource != null) {
            return defaultResource;
        } else {
            return loadResource();
        }
    }

    public abstract String getSprite();

    public abstract String getSprite(E key);

    public Optional<Class<E>> getStates() {
        return Optional.ofNullable(enumClass);
    }
}
